import {
  S3Client,
  ListObjectsV2Command,
  GetObjectCommand,
} from "@aws-sdk/client-s3";
import { getSignedUrl } from "@aws-sdk/s3-request-presigner";

// Initialize S3 client with environment variables
const s3ClientOptions = {
  region: import.meta.env.S3_REGION,
  credentials: {
    accessKeyId: import.meta.env.S3_ACCESS_KEY,
    secretAccessKey: import.meta.env.S3_SECRET_KEY,
  },
};

// Add endpoint URL for S3-compatible services like Backblaze B2
if (import.meta.env.S3_ENDPOINT) {
  s3ClientOptions.endpoint = import.meta.env.S3_ENDPOINT;

  // For some S3-compatible services, we need to force path style addressing
  if (import.meta.env.S3_FORCE_PATH_STYLE === "true") {
    s3ClientOptions.forcePathStyle = true;
  }
}

const s3Client = new S3Client(s3ClientOptions);
const bucketName = import.meta.env.S3_BUCKET_NAME;

/**
 * List images from S3 bucket with pagination
 * @param {string} continuationToken - Token for pagination
 * @param {number} maxKeys - Maximum number of keys to return
 * @returns {Promise<{images: Array, nextContinuationToken: string}>}
 */
export async function listImages(continuationToken = undefined, maxKeys = 20) {
  const command = new ListObjectsV2Command({
    Bucket: bucketName,
    MaxKeys: maxKeys,
    ContinuationToken: continuationToken,
    Prefix: "Camera/",
  });

  try {
    const response = await s3Client.send(command);

    // Filter for image files only (common image extensions)
    const imageExtensions = [".jpg", ".jpeg", ".png", ".gif", ".webp", ".heic"];
    const images =
      response.Contents?.filter((item) =>
        imageExtensions.some((ext) => item.Key.toLowerCase().endsWith(ext))
      ) || [];

    // Generate signed URLs for each image
    const imagesWithUrls = await Promise.all(
      images.map(async (image) => {
        const getObjectCommand = new GetObjectCommand({
          Bucket: bucketName,
          Key: image.Key,
        });

        // Create a signed URL that expires in 1 hour
        const url = await getSignedUrl(s3Client, getObjectCommand, {
          expiresIn: 3600,
        });

        return {
          key: image.Key,
          url,
          lastModified: image.LastModified,
          size: image.Size,
        };
      })
    );

    return {
      images: imagesWithUrls,
      nextContinuationToken: response.NextContinuationToken,
    };
  } catch (error) {
    console.error("Error listing images from S3:", error);
    throw error;
  }
}
